<?php
//
// +---------------------------------------------------------------------------+
// | Metatags Plugin for Geeklog - The Ultimate Weblog                         |
// +---------------------------------------------------------------------------+
// | geeklog/plugins/metatags/functions.inc                                    |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 mystral-kk - geeklog AT mystral-kk DOT net             |
// |                                                                           |
// | Constructed with the Universal Plugin                                     |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

if (strpos(strtolower($_SERVER['PHP_SELF']), 'functions.inc') !== FALSE) {
	die('This file can not be used on its own.');
}

/**
* Language file Include
*/
$langfile = $_CONF['path'] . 'plugins/metatags/language/' . $_CONF['language'] . '.php';
clearstatcache();
if (file_exists($langfile)) {
	include_once $langfile;
} else {
	include_once $_CONF['path'] . 'plugins/metatags/language/english.php';
}

/**
* Load the plugin configuration
*/
global $_METATAGS_CONF;

require_once $_CONF['path'] . 'plugins/metatags/config.php';
METATAGS_loadConfig();

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementations                                        |
// +---------------------------------------------------------------------------+

/**
* Returns the version for this plugin
*
* @return string VersionNo
*/
function plugin_chkVersion_metatags() {
	global $_METATAGS_CONF;
	
	return $_METATAGS_CONF['pi_version'];
}

/**
* Upgrade the plugin
*
* Universal Plugin:  modify/uncomment use it
*
* @return   boolean TRUE (= success)
*/
function plugin_upgrade_metatags() {
	global $_CONF, $_TABLES, $_METATAGS_CONF;
	
	$result = DB_query("SELECT pi_version FROM {$_TABLES['plugins']} WHERE (pi_name = 'metatags')");
	$A = DB_fetchArray($result);
	$pi_version = $A['pi_version'];		// Version info stored in database
	
	// Do upgrading
	
	switch ($pi_version) {
		case '1.0.0':
		case '1.0.1':
		case '1.0.2':
			$me = 'metatags';
		    $c = config::get_instance();
			$c->add('sp_php', $_METATAGS_CONF['sp_php'], 'select', 0, 0, 1, 30, TRUE, $me);
			break;
	}
	
	// Update plugin info
	
	$result = DB_query("UPDATE {$_TABLES['plugins']} SET pi_version = \"{$_METATAGS_CONF['pi_version']}\" WHERE (pi_name = 'metatags')");
	COM_errorLog(ucfirst("metatags") . " plugin was successfully updated to version {$_METATAGS_CONF['pi_version']}.");
	
	return TRUE;
}

/**
* Automatic uninstall function for plugins
*
* @return   array
*
* This code is automatically uninstalling the plugin.
* It passes an array to the core code function that removes
* tables, groups, features and php blocks from the tables.
* Additionally, this code can perform special actions that cannot be
* foreseen by the core code (interactions with other plugins for example)
*/
function plugin_autouninstall_metatags() {
	global $_METATAGS_CONF;
	
	$out = array (
		/* give the name of the tables, without $_TABLES[] */
		'tables' => array(),
		/* give the full name of the group, as in the db */
		'groups' => array('Metatags Admin'),
		/* give the full name of the feature, as in the db */
		'features' => $_METATAGS_CONF['FEATURES'],
		/* give the full name of the block, including 'phpblock_', etc */
		'php_blocks' => array(),
		/* give all vars with their name */
		'vars'=> array()
	);
	return $out;
}

/**
* Loads config infor with config.class.php
*/
function METATAGS_loadConfig() {
	global $_CONF, $_METATAGS_CONF;
	
	require_once $_CONF['path_system'] . 'classes/config.class.php';
	$config = config::get_instance();
	if ($config->group_exists('metatags')) {
		$temp = $config->get_config('metatags');
		if (is_array($temp) AND (count($temp) > 0)) {
			$_METATAGS_CONF = array_merge($_METATAGS_CONF, $temp);
		}
	}
}

/**
* Returns a string escaped for HTML output
*/
function METATAGS_esc($str) {
	static $encoding = NULL;
	
	if ($encoding === NULL) {
		$encoding = COM_getCharset();
	}
	
	$str = str_replace(
		array('&lt;', '&gt;', '&amp;', '&quot;', '&#039;'),
		array(   '<',    '>',     '&',      '"',      "'"),
		$str
	);
	return htmlspecialchars($str, ENT_QUOTES, $encoding);
}

/**
* Returns a lang var
*/
function METATAGS_str($index, $noesc = FALSE) {
	global $LANG_METATAGS;
	
	if (isset($LANG_METATAGS[$index])) {
		if ($noesc) {
			return $LANG_METATAGS[$index];
		} else {
			return METATAGS_esc($LANG_METATAGS[$index]);
		}
	} else {
		COM_errorLog('METATAGS_str: undefined index: "' . $index . '"');
		return '(undefined)';
	}
}

/**
* Return the content of a static page WITHOUT replacing autotags
* Code taken from SP_render_content()
*
* @param  id  string  staticpage id
* @return     string  page content
*/
function METATAGS_getStaticpage($id) {
	global $_CONF, $_TABLES, $_SP_CONF, $_METATAGS_CONF;
	
	$retval = '';
	
	$sql = "SELECT sp_content, sp_php "
		 . "FROM {$_TABLES['staticpage']} "
		 . "WHERE " . SP_getPerms()
		 . " AND (sp_id = '" . addslashes($id) . "') ";
	
	$result = DB_query($sql);
	
	if (!DB_error() AND (DB_numRows($result) == 1)) {
		$A = DB_fetchArray($result);
		$sp_content = stripslashes($A['sp_content']);
		$sp_php     = $A['sp_php'];
		
		if ($_METATAGS_CONF['sp_php']) {
			if ($_SP_CONF['allow_php'] == 1) {
				// Check for type (ie html or php)
				if ($sp_php == 1) {
					$retval = @eval($sp_content);
				} else if ($sp_php == 2) {
					ob_start();
					@eval($sp_content);
					$retval = ob_get_contents();
					ob_end_clean();
				} else {
					$retval = $sp_content;
				}
			} else {
				if ($sp_php != 0) {
					COM_errorLog ("PHP in static pages is disabled.  Can not display page '$id'.", 1);
				} else {
					$retval = $sp_content;
				}
			}
		} else {
			$retval = $sp_content;
		}
	}
	
	return $retval;
}

/**
* Return <meta> tags
*/
function plugin_getheadercode_metatags() {
	global $_CONF, $_TABLES, $_METATAGS_CONF;
	
	$retval = '';
	$type   = '';
	
	// Check the current URL for ids
	$uri = strtolower(COM_getCurrentURL());
	
	if ($_CONF['url_rewrite']) {
		if (preg_match("|article\.php/(.+?)(/print)?$|i", $uri, $match)) {
			$type = 'article';
		} else if (preg_match("|staticpages/index\.php/(.+?)(/print)?$|i", $uri, $match)) {
			$type = 'staticpages';
		}
	} else {
		if (preg_match("|article\.php\?story=(.+?)(&mode=print)?$|i", $uri, $match)) {
			$type = 'article';
		} else if (preg_match("|staticpages/index\.php\?page=(.+?)(&disp_mode=print)?$|i", $uri, $match)) {
			$type = 'staticpages';
		}
	}
	
	$id = '';
	if (isset($match[1])) {
		$id = COM_applyFilter($match[1]);
		if ($id == '') {
			$type= '';
		}
	}
	
	// Retrieve the relevant content from DB
	$target = '';
	
	switch ($type) {
		case 'article':
			$sql = "SELECT introtext, bodytext "
				 . "FROM {$_TABLES['stories']} "
				 . "WHERE (sid = '" . addslashes($id) . "') "
				 . COM_getPermSQL('AND')
				 . COM_getTopicSQL('AND');
			$result = DB_query($sql);
			if (!DB_error() AND (DB_numRows($result) == 1)) {
				$A = DB_fetchArray($result);
				$target = stripslashes($A['introtext'] . ' ' . $A['bodytext']);
			}
				 
			break;
		
		case 'staticpages':
			$target = METATAGS_getStaticpage($id);
			break;
		
		default:
			// Do nothing
			break;
	}
	
	$tags = array();
	
	// Check if the content has the autotag
	if ($target != '') {
		$autotag = '|\[' . preg_quote($_METATAGS_CONF['tagname'], '|')
				 . '\:([^\]]*)\]|i';
		$num    = preg_match_all($autotag, $target, $match);
		$tokens = array_keys($_METATAGS_CONF['replace']);
		
		if ($num > 0) {
			foreach ($match[1] as $m) {
				list($key, $value) = explode(' ', $m, 2);
				$key   = strtolower($key);
				$value = trim($value);
				if (in_array($key, $tokens) AND ($value != '')) {
					$tags[$_METATAGS_CONF['replace'][$key]] = $value;
				}
			}
		}
	}
	
	if (!isset($tags['keywords']) AND isset($_METATAGS_CONF['keywords'])) {
		$tags['keywords'] = $_METATAGS_CONF['keywords'];
	}
	
	if (!isset($tags['description']) AND isset($_METATAGS_CONF['description'])) {
		$tags['description'] = $_METATAGS_CONF['description'];
	}
	
	if (count($tags) > 0) {
		// Convert autotags into <meta> headers
		foreach ($tags as $key => $value) {
			$retval .= '<meta name="' . $key . '" content="'
					.  METATAGS_esc($value) . '"' . XHTML . '>' . LB;
		}
	}
	
	if ($retval != '') {
		$retval = LB . $retval;
	}
	
	return $retval;
}

/**
* Implements the [meta:] autotag.
*
*/
function plugin_autotags_metatags($op, $content = '', $autotag = '') {
	global $_CONF, $_METATAGS_CONF;
	
	if ($op == 'tagname' ) {
		return $_METATAGS_CONF['tagname'];
	} else if ($op == 'parse') {
		// Let editors see the autotag but hide them for non-editors
		if (!SEC_hasRights('story.edit')) {
			$content = str_replace($autotag['tagstr'], '', $content);
		}
		
		return $content;
	}
}
